import os.path
import re
import sys
import difflib
import tarfile
import numpy

extensions = '\.(html|php|sql|xml|css)$'

path1_dict = {}
path2_dict = {}

results = {
    'commonDirs': 0, #   number of common directories
    'path1OnlyDirs': 0, # number of directories only in path1
    'path2OnlyDirs': 0, # number of directories only in path2
    'commonFiles': 0, # number of common files (modified and unmodified)
    'commonEqualFiles': 0, # number of common (unmodified) files
    'path1OnlyFiles': 0, # number of files only in path1
    'path2OnlyFiles': 0, # number of files only in path2
    'commonElements': 0, # number of common elements
    'totalElements': 0 # number of total elements
    }

'''
Asi es como calcula el ratio internamente difflib

def _calculate_ratio(matches, length):
    if length:
        return 2.0 * matches / length
    return 1.0

    def ratio(self):
        matches = reduce(lambda sum, triple: sum + triple[-1],
                         self.get_matching_blocks(), 0)
        return _calculate_ratio(matches, len(self.a) + len(self.b))
'''

def calculateSimilarity(diffElement):
    matches = reduce(lambda sum, triple: sum + triple[-1], diffElement.get_matching_blocks(), 0)
    results['commonElements'] += matches
    sizeA = len(diffElement.a)
    results['totalElements'] += sizeA
    if sizeA:
        similarity = matches / float(sizeA)
    else:
        similarity = 1.0

    if similarity == 1.0:
        results['commonEqualFiles'] += 1
    elif similarity:
        results['commonFiles'] += 1
    else: # 0.0 similarity
        results['path1OnlyFiles'] += 1
        results['path2OnlyFiles'] += 1


def determine_path(path):
    if path[-7:] == '.tar.gz' or path[-4:] == '.tgz':
        tar = tarfile.open(path)
        if '/' in path:
            extracting_path = path[:path.rfind('/')]
        else:
            extracting_path = "."
#        print "Extracting", path, "tarball in following path", extracting_path
        tarInfo = tar.next()
        if tarInfo.isdir():
          name = tarInfo.name
          if (not(os.path.exists(extracting_path+name))):
            tar.extractall(extracting_path)
        tar.close()
        return path[:-7] + '/' if path[-7:] == '.tar.gz' else path[:-4] + '/'
    else:
        return path if path[-1] == '/' else path + '/'


def visit(args, dirname, names):
    (path, dictionary) = args
    files = [elem for elem in names if re.search(extensions, elem)]
    if files:
        dictionary[dirname.replace(path, "")] = files


def compare_files(directory, filename):
    filepath1 = path1 + directory + '/' + filename
    filepath2 = path2 + directory + '/' + filename
    calculateSimilarity(difflib.SequenceMatcher(None,open(filepath1).readlines(), open(filepath2).readlines()))


def insertDifferentFile (directory, list1):
    for element in list1:
        results['path1OnlyFiles'] += 1 
        filepath1 = path1 + directory + '/' + element
        diffElement = difflib.SequenceMatcher(None,open(filepath1).readlines(), open(filepath1).readlines())
        results['totalElements'] += len(diffElement.a)


def compare_lists(list1, list2, directory):
    set1 = set(list1)
    set2 = set(list2)

    results['path2OnlyFiles'] += len(set2.difference(set1))

    insertDifferentFile(directory, list(set1.difference(set2)))
    

    for element in list(set2.intersection(set1)):
        compare_files(directory, element)


def compare_dicts(dict1, dict2):
    set1 = set(dict1)
    set2 = set(dict2)
    
    onlyDirs1 = set1.difference(set2)
    onlyDirs2 = set2.difference(set1)

    intersectionDirs = set1.intersection(set2)

    results['path1OnlyDirs'] += len(onlyDirs1)
    results['path2OnlyDirs'] += len(onlyDirs2)
    results['commonDirs'] += len(intersectionDirs)
    
    for directory in list(onlyDirs1):
        insertDifferentFile(directory, dict1[directory])

    for directory in list(onlyDirs2):
        results['path2OnlyFiles'] += len(dict2[directory])
    
    for directory in list(intersectionDirs):
#        print "+A", directory
        compare_lists(dict1[directory], dict2[directory], directory)

def clean_path(path):
    if '/' in path:
        path = path[path.rfind('/')+1:]
    if path[-7:] == '.tar.gz':
        path = path[:-7]
    elif path[-4:] == '.tgz':    
        path = path[:-4]
    return path


def print_results(dictionary, path1, path2):
    path1 = clean_path(path1)
    path2 = clean_path(path2)
    print "============================================================"
    print "Project A:", path1
    print "Project B:", path2
    print "Common dirs:", dictionary['commonDirs']
    print "Dirs only in A:", dictionary['path1OnlyDirs']
    print "Dirs only in B:", dictionary['path2OnlyDirs']
    print "Common files:", dictionary['commonEqualFiles'] + dictionary['commonFiles']
    print "  Equal files:", dictionary['commonEqualFiles']
    print "  Non-equal files:", dictionary['commonFiles']
    print "Average:", dictionary['commonElements']/float(dictionary['totalElements'])
    print "Files only in A:", dictionary['path1OnlyFiles']
    print "Files only in B:", dictionary['path2OnlyFiles']

path1 = determine_path(sys.argv[1])
path2 = determine_path(sys.argv[2])
os.path.walk(path1, visit, (path1, path1_dict))
os.path.walk(path2, visit, (path2, path2_dict))
compare_dicts(path1_dict, path2_dict)
print_results(results, sys.argv[1], sys.argv[2])
